﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace DeliciousDotNet.OfflineReader
{
	class RecipeRepository
	{
		public ObservableCollection<Recipe> Recipes
		{
			get
			{
				if (recipes == null)
				{
					RebuildObservableCollectionFrom(context.Recipes);
				}
				return recipes;
			}
		}

		public void FilterByName(string name)
		{
			var filteredRecipes = context.Recipes.Where(r => r.Name.ToLower().Contains(name.ToLower()));
			RebuildObservableCollectionFrom(filteredRecipes);
		}

		public void FilterByIngredients(string[] requestedIngredients)
		{
			IEnumerable<Recipe> filteredRecipes =
				context.Recipes.ToList().
				Where(recipe => requestedIngredients.All(
								requestedIngredient => recipe.Ingredients.Any(
									recipeIngredient => recipeIngredient.Description.Trim().ToLower().Contains(requestedIngredient.Trim().ToLower()))));
			RebuildObservableCollectionFrom(filteredRecipes);
		}

		void RebuildObservableCollectionFrom(IEnumerable<Recipe> newRecipes)
		{
			if (recipes == null)
			{
				recipes = new ObservableCollection<Recipe>();
			}
			else
			{
				recipes.Clear();
			}

			foreach (Recipe r in newRecipes)
			{
				recipes.Add(r);
			}
		}

		ObservableCollection<Recipe> recipes;
		DeliciousDataContext context = new DeliciousDataContext();
	}
}

namespace DeliciousDotNet.OfflineReader.Testing
{
	using NUnit.Framework;

	[TestFixture()]
	public class RecipeRepositoryTest
	{
		[Test]
		public void TestWithNoFiltering()
		{
			Assert.AreEqual(2, repository.Recipes.Count);
		}

		[Test]
		public void TestWithNameFiltering()
		{
			repository.FilterByName("lamb");
			Assert.AreEqual(1, repository.Recipes.Count);
			Assert.AreEqual("Rosemary Lamb Chops", repository.Recipes.First().Name);
		}

		[Test]
		public void TestWithIngredientFiltering()
		{
			repository.FilterByIngredients(new string[] {"butter", "lemon"} );
			Assert.AreEqual(2, repository.Recipes.Count);

			repository.FilterByIngredients(new string[] { "butter", "veal" } );
			Assert.AreEqual(1, repository.Recipes.Count);
			Assert.AreEqual("Wiener Schnitzel", repository.Recipes.First().Name);
		}

		[SetUp]
		public void Setup()
		{
			RecipeUpdater updater = new RecipeUpdater();
			updater.RemoveAll();
			updater.LoadFromXml();

			repository = new RecipeRepository();
		}

		RecipeRepository repository;

	}
}
